'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getRockeyHoc = undefined;

var _templateObject = _taggedTemplateLiteral([''], ['']);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _isString = require('lodash/isString');

var _isString2 = _interopRequireDefault(_isString);

var _isArray = require('lodash/isArray');

var _isArray2 = _interopRequireDefault(_isArray);

var _rule = require('rockey/rule');

var _rule2 = _interopRequireDefault(_rule);

var _look = require('./look');

var _look2 = _interopRequireDefault(_look);

var _RockeyHocWithHandlers = require('./utils/RockeyHocWithHandlers');

var _RockeyHocWithHandlers2 = _interopRequireDefault(_RockeyHocWithHandlers);

var _handler = require('./handler');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _taggedTemplateLiteral(strings, raw) { return Object.freeze(Object.defineProperties(strings, { raw: { value: Object.freeze(raw) } })); }

var COMPONENT_EXTENDS = 'COMPONENT_EXTENDS';
var DEFINE_COMPONENT_NAME = 'DEFINE_COMPONENT_NAME';
var WAS_CALLED_AS_REACT_COMPONENT = 'WAS_CALLED_AS_REACT_COMPONENT';

var getCallType = function getCallType() {
  if (arguments.length === 1 && (0, _isString2.default)(arguments.length <= 0 ? undefined : arguments[0])) {
    return DEFINE_COMPONENT_NAME;
  } else if ((0, _isArray2.default)(arguments.length <= 0 ? undefined : arguments[0])) {
    return COMPONENT_EXTENDS;
  } else {
    return WAS_CALLED_AS_REACT_COMPONENT;
  }
};

var anonymysRockeyCounter = 0;
var childCounter = {};

var createEmtpyCss = function createEmtpyCss(name) {
  var css = (0, _rule2.default)(_templateObject);
  css.wrapWith(name);
  return css;
};

var getRockeyHoc = exports.getRockeyHoc = function getRockeyHoc() {
  var RockeyHoc = function RockeyHoc(BaseComponent) {
    var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
        displayName = _ref.displayName,
        parentName = _ref.parentName,
        css = _ref.css,
        at = _ref.at;

    var name = displayName || 'AnonymysRockey' + ++anonymysRockeyCounter;
    var queuedMixins = null;

    var FlexibleRockeyHoc = function FlexibleRockeyHoc() {
      var CALL_TYPE = getCallType.apply(undefined, arguments);

      switch (CALL_TYPE) {
        case DEFINE_COMPONENT_NAME:
          var childComponentName = arguments.length <= 0 ? undefined : arguments[0];

          return RockeyHoc(BaseComponent, {
            displayName: childComponentName,
            parentName: name,
            css: css,
            at: DEFINE_COMPONENT_NAME
          });

        case COMPONENT_EXTENDS:
          var componentCss = _rule2.default.apply(undefined, arguments);

          if (queuedMixins) {
            componentCss.addMixins(queuedMixins);
          }

          if (css) {
            componentCss.addParent(css);
          }

          // TODO: ?
          var childName = null;

          if (parentName && at !== DEFINE_COMPONENT_NAME) {
            if (!childCounter[parentName]) {
              childCounter[parentName] = 0;
            }

            childName = 'Child' + parentName + '-' + ++childCounter[parentName];
          } else {
            childName = name;
          }

          componentCss.wrapWith(childName);

          return RockeyHoc(BaseComponent, {
            displayName: childName,
            parentName: name,
            css: componentCss,
            at: COMPONENT_EXTENDS
          });

        case WAS_CALLED_AS_REACT_COMPONENT:
          var props = arguments.length <= 0 ? undefined : arguments[0];

          if (!css) {
            css = createEmtpyCss(name);
          }

          var finshCssRule = null;
          if (at === DEFINE_COMPONENT_NAME) {
            // wrap with name because Function is used as React comopnent
            // but currect css object === parentCss. line :60
            finshCssRule = createEmtpyCss(name);
            finshCssRule.addParent(css);
          } else {
            finshCssRule = css;
          }

          // collect handler mixins
          var handlers = [];
          Object.keys(finshCssRule.mixins).forEach(function (key) {
            var mixin = finshCssRule.mixins[key];

            if (mixin[_handler.ROCKEY_MIXIN_HANDLER_KEY]) {
              handlers.push(mixin);
            }
          });

          if (handlers.length) {
            return _react2.default.createElement(_RockeyHocWithHandlers2.default, {
              css: finshCssRule,
              selector: name,
              handlers: handlers,
              BaseComponent: BaseComponent,
              proxy: props
            });
          } else {
            var classList = finshCssRule.getClassList(props);
            var className = (0, _classnames2.default)(classList[name], props.className);

            return _react2.default.createElement(BaseComponent, Object.assign({}, props, {
              className: className
            }));
          }

        default:
          throw new Error('Wrong component call');
      }
    };

    FlexibleRockeyHoc.displayName = 'Rockey(' + name + ')';

    FlexibleRockeyHoc.extends = function (displayName, childCss) {
      if (!css) {
        css = createEmtpyCss(name);
      }

      childCss.addParent(css);

      return RockeyHoc(BaseComponent, {
        displayName: displayName,
        parentName: name,
        css: childCss
      });
    };

    FlexibleRockeyHoc.look = function () {
      var component = RockeyHoc(BaseComponent, {
        displayName: displayName,
        parentName: parentName,
        css: css
      });

      var childComponents = (0, _look2.default)(component, {
        extendBase: false
      }).apply(undefined, arguments);

      Object.keys(childComponents).forEach(function (key) {
        FlexibleRockeyHoc[key] = childComponents[key];
      });

      return childComponents;
    };

    // TODO: create css object instead of queuedMixns
    FlexibleRockeyHoc.addMixins = function (mixins) {
      if (css) {
        css.addMixins(mixins);
      } else {
        queuedMixins = mixins;
      }
    };

    FlexibleRockeyHoc.propTypes = BaseComponent.propTypes;
    FlexibleRockeyHoc.defaultProps = BaseComponent.defaultProps;

    return FlexibleRockeyHoc;
  };

  return RockeyHoc;
};

var RockeyHoc = getRockeyHoc();
exports.default = RockeyHoc;